# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx_xml2rfc documentation config."""

from __future__ import annotations

import datetime
import importlib.metadata
import os
import sys

import sphinx_readable_theme

import sphinx_xml2rfc

_dist = importlib.metadata.distribution(sphinx_xml2rfc.__name__)
_buildtime = datetime.datetime.utcnow()

# -- Project Information

project = _dist.metadata["name"]
author = _dist.metadata["author"]

_from_year = 2021
_to_year = _buildtime.year
if _from_year < _to_year:
    _year_range = f"{_from_year}-{_to_year}"
else:
    _year_range = f"{_from_year}"
copyright = f"{_year_range}, {author}"

release = _dist.version
version = ".".join(release.split(".")[:2])

# -- General configuration

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             "_extensions")))
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "myst_parser",
]
source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}
exclude_patterns = []

# -- HTML output

html_theme = "readable"
html_theme_path = [sphinx_readable_theme.get_html_theme_path()]

# -- Autodoc configuration

autodoc_member_order = "bysource"
autodoc_typehints = "description"
autodoc_typehints_description_target = "all"


# -- Intersphinx configuration

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
}

# -- Markdown processing

myst_enable_extensions = [
    "colon_fence",
]



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx_xml2rfc test configuration."""

from __future__ import annotations

import datetime
import os
from unittest.mock import NonCallableMagicMock, PropertyMock, patch

import git

import pytest

from sphinx.testing.path import path

AUTOGEN_TEST_DRAFT = "test-draft"

pytest_plugins = 'sphinx.testing.fixtures'
collect_ignore = ["docs"]


@pytest.fixture(scope="session")
def rootdir():
    """Set the root directory of test sources."""
    return path(__file__).parent.abspath() / "roots"


@pytest.fixture(scope="session")
def test_draft_name():
    """Provide test draft name."""
    return "test-draft"


@pytest.fixture
def patched_app(test_draft_name, test_params, app_params,
                make_app, shared_result):
    """Provide a sphinx application with `git` patched."""

    def blob_stream_data(fd):
        """Write bytes from dummy draft."""
        with open(os.path.join(os.path.dirname(__file__),
                  f"{test_draft_name}.xml")) as f:
            template = f.read()
        text = template.format(version_path=fd.name)
        fd.write(text.encode())

    with patch("git.Repo", autospec=True) as MockRepo:  # noqa: N806
        mock_repo = MockRepo.return_value

        mock_blob = NonCallableMagicMock(spec=git.Blob)
        mock_blob.name = "test-draft.xml"
        mock_blob.stream_data.side_effect = blob_stream_data

        ref_specs = {"branches": {"name": "main",
                                  "path": "refs/heads/main",
                                  "repo": mock_repo},
                     "tags": {"name": "v0",
                              "path": "refs/tags/v0",
                              "repo": mock_repo}}
        mock_refs = list()
        for prop, ref_attrs in ref_specs.items():
            mock_ref = NonCallableMagicMock(spec=git.Reference)
            mock_ref.configure_mock(**ref_attrs)
            mock_ref.commit.committed_datetime = datetime.datetime.utcnow()
            type(mock_ref.commit.tree).blobs = PropertyMock(return_value=[mock_blob])  # noqa: E501
            setattr(type(mock_repo), prop, PropertyMock(return_value=[mock_ref]))  # noqa: E501
            mock_refs.append(mock_ref)

        args, kwargs = app_params
        app_ = make_app(*args, **kwargs)
        yield (app_, MockRepo, mock_refs, mock_blob)

        print('# testroot:', kwargs.get('testroot', 'root'))
        print('# builder:', app_.builder.name)
        print('# srcdir:', app_.srcdir)
        print('# outdir:', app_.outdir)
        print('# status:', '\n' + app_._status.getvalue())
        print('# warning:', '\n' + app_._warning.getvalue())

        if test_params['shared_result']:
            shared_result.store(test_params['shared_result'], app_)



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""Sphinx configuration for test_domain test."""

from __future__ import annotations

extensions = ["sphinx_xml2rfc"]



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx-xml2rfc domain module."""

from __future__ import annotations

import difflib
import itertools
import os
import typing

import docutils

import sphinx

from .utils import get_base_dir

if typing.TYPE_CHECKING:
    import sphinx.application
    import sphinx.directives
    import sphinx.domains

logger = sphinx.util.logging.getLogger(__name__)


class VersionDirective(sphinx.directives.ObjectDescription[str]):
    """RST xml2rfc:version directive."""

    has_content = False
    required_arguments = 1
    option_spec = {
        "ref_type": docutils.parsers.rst.directives.unchanged_required,
        "ref_name": docutils.parsers.rst.directives.unchanged_required,
        "ref_path": docutils.parsers.rst.directives.unchanged_required,
    }

    def run(self) -> typing.List[docutils.nodes.Node]:
        """Save the version object for later processing."""
        self.version = Version(base_dir=get_base_dir(self.env.app),
                               draft=self.arguments[0].strip(),
                               ref_type=self.options.get("ref_type").strip(),
                               ref_name=self.options.get("ref_name").strip(),
                               ref_path=self.options.get("ref_path").strip())
        return super().run()

    def handle_signature(self, sig: str,
                         signode: sphinx.addnodes.desc_signature) -> str:
        """Handle the object signature."""
        signode += sphinx.addnodes.desc_name(text=sig)
        return sig

    def transform_content(self,
                          contentnode: sphinx.addnodes.desc_content) -> None:
        """Read the version source file and insert into the node."""
        src = self.version.read_src()
        code_block = docutils.nodes.literal_block(src, src)
        code_block["language"] = "text"
        contentnode += code_block

    def add_target_and_index(self, name: str, sig: str,
                             signode: sphinx.addnodes.desc_signature) -> None:
        """Add ourself to the domain."""
        signode["ids"].append(self.version.anchor)
        domain = self.env.get_domain("xml2rfc")
        typing.cast(Xml2rfcDomain, domain).add_version(self.version)


class Version(typing.NamedTuple):
    """Pickable object for storing version instance data."""

    base_dir: str
    draft: str
    ref_type: str
    ref_name: str
    ref_path: str

    @property
    def anchor(self) -> str:
        """Generate anchor for version node."""
        return f"xml2rfc-version-{self.draft}-{self.ref_type}-{self.ref_name}"

    def read_src(self) -> str:
        """Read draft text from file."""
        src_path = os.path.join(self.base_dir,
                                self.ref_path,
                                f"{self.draft}.txt")
        logger.info(f"reading internet draft content from {src_path}")
        with open(src_path) as fd:
            src = fd.read()
        return src


class VersionNotFound(Exception):
    """A version with a matching 'ref.path' could not be found."""

    pass


class DiffDirective(sphinx.directives.ObjectDescription[str]):
    """RST xml2rfc:diff directive."""

    has_content = False
    required_arguments = 1
    option_spec = {
        "from": docutils.parsers.rst.directives.unchanged_required,
        "to": docutils.parsers.rst.directives.unchanged_required,
    }

    def run(self) -> typing.List[docutils.nodes.Node]:
        """Save the version object for later processing."""
        self.diff = Diff(draft=self.arguments[0].strip(),
                         ref_from=self.options.get("from").strip(),
                         ref_to=self.options.get("to").strip())
        return super().run()

    def handle_signature(self, sig: str,
                         signode: sphinx.addnodes.desc_signature) -> str:
        """Handle the object signature."""
        signode += sphinx.addnodes.desc_name(text=sig)
        return sig

    def transform_content(self,
                          contentnode: sphinx.addnodes.desc_content) -> None:
        """Insert a placeholder node for later resolution."""
        target = f"{self.diff.ref_from}::{self.diff.ref_to}"
        refnode = sphinx.addnodes.pending_xref("", refdomain="xml2rfc",
                                               reftype="diff",
                                               reftarget=target)
        temp_msg = f"diff targets {target} not resolved"
        temp = docutils.nodes.literal(temp_msg, temp_msg)
        refnode += temp
        contentnode += refnode

    def add_target_and_index(self, name: str, sig: str,
                             signode: sphinx.addnodes.desc_signature) -> None:
        """Add ourself to the domain."""
        signode["ids"].append(self.diff.anchor)
        domain = self.env.get_domain("xml2rfc")
        typing.cast(Xml2rfcDomain, domain).add_diff(self.diff)


class Diff(typing.NamedTuple):
    """Pickable object for storing diff instance data."""

    draft: str
    ref_to: str
    ref_from: str

    @property
    def anchor(self) -> str:
        """Generate anchor for version node."""
        return f"xml2rfc-diff-{self.draft}-{self.ref_from}-{self.ref_to}"


class VersionsIndex(sphinx.domains.Index):
    """Sphinx object index for draft versions."""

    name = "version-index"
    localname = "Internet Draft Versions"
    shortname = "drafts"

    _Items = typing.Tuple[
        typing.List[
            typing.Tuple[
                str,
                typing.List[sphinx.domains.IndexEntry],
            ],
        ],
        bool,
    ]

    def generate(self,
                 docnames: typing.Optional[typing.Iterable[str]] = None) -> _Items:  # noqa: E501
        """Build the index from obects stored on the domain."""

        def by_ref_type(version: typing.Tuple[str, str, str,
                                              str, str, int]) -> str:
            return version[2]

        versions = self.domain.get_objects()
        index = [(group, [sphinx.domains.IndexEntry(name, 0, docname, anchor,
                                                    "", "", object_type)
                          for name, _, object_type, docname, anchor, _
                          in items])
                 for group, items in itertools.groupby(versions, by_ref_type)]
        logger.debug(f"{index=}")
        return index, True


class Xml2rfcDomain(sphinx.domains.Domain):
    """Sphinx language domain for draft versions and diffs."""

    name = "xml2rfc"
    roles = {"ref": sphinx.roles.XRefRole()}
    directives = {"version": VersionDirective,
                  "diff": DiffDirective}
    indices = [VersionsIndex]

    initial_data: typing.Dict[typing.Any, typing.Any] = {"versions": set(),
                                                         "diffs": set()}

    def get_objects(self) -> typing.Iterable[typing.Tuple[str, str, str,
                                                          str, str, int]]:
        """Iterate over objects to construct indices."""
        for version, docname in self.data["versions"]:
            name = display_name = f"{version.draft}@{version.ref_name}"
            object_type = self.object_type_name(version.ref_type)
            anchor = version.anchor
            priority = 1
            yield (name, display_name, object_type, docname, anchor, priority)

    @property
    def versions(self) -> typing.Iterator[Version]:
        """Iterate over versions."""
        for version, _ in self.data["versions"]:
            yield version

    def search_version(self, ref_path: str) -> Version:
        """Get a draft version by ref_path."""
        for version in self.versions:
            if version.ref_path == ref_path:
                return version
        raise VersionNotFound(ref_path)

    def resolve_xref(self, env: sphinx.environment.BuildEnvironment,
                     fromdocname: str, builder: sphinx.builders.Builder,
                     typ: str, target: str, node: sphinx.addnodes.pending_xref,
                     contnode: docutils.nodes.Element) -> typing.Optional[docutils.nodes.Element]:  # noqa: E501
        """Resolve pending xrefs."""
        if typ == "diff":
            return self.construct_diff(target)
        return None

    def construct_diff(self,
                       target: str) -> docutils.nodes.Element:
        """Populate diff node content."""
        (ref_from, ref_to) = target.split("::")
        from_version = self.search_version(ref_from)
        to_version = self.search_version(ref_to)
        diff_lines = difflib.unified_diff(from_version.read_src().split("\n"),
                                          to_version.read_src().split("\n"),
                                          fromfile=from_version.ref_path,
                                          tofile=to_version.ref_path)
        container = docutils.nodes.container()
        if diff_lines:
            desc_src = f"Changes {ref_from} ⟼ {ref_to}"
            desc = docutils.nodes.paragraph(desc_src, desc_src)
            diff_src = "\n".join(diff_lines)
            code_block = docutils.nodes.literal_block(diff_src, diff_src)
            code_block["language"] = "diff"
            container += [desc, code_block]
        else:
            desc_src = f"No changes {ref_from} ⟼ {ref_to}"
            desc = docutils.nodes.paragraph(desc_src, desc_src)
            container += desc
        return container

    @staticmethod
    def object_type_name(ref_type: str) -> str:
        """Construct object type names on the fly."""
        base_name = "Internet Draft Version"
        names = {"branches": "Branch",
                 "tags": "Tag"}
        try:
            return f"{base_name} ({names[ref_type]})"
        except KeyError:
            return base_name

    def add_version(self, version: Version) -> None:
        """Add a version to the domain."""
        self.data["versions"].add((version, self.env.docname))

    def add_diff(self, diff: Diff) -> None:
        """Add a diff to the domain."""
        self.data["diffs"].add((diff, self.env.docname))



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx-xml2rfc autogen module."""

from __future__ import annotations

import datetime
import itertools
import os
import re
import subprocess  # noqa: S404
import tempfile
import typing

import git

import sphinx

from .utils import get_base_dir

if typing.TYPE_CHECKING:
    import sphinx.application

logger = sphinx.util.logging.getLogger(__name__)

Refs = typing.Dict[str, typing.Dict[str, git.refs.reference.Reference]]


def autogen_run(app: sphinx.application.Sphinx) -> None:
    """Generate text output from XML sources at every git ref."""
    if app.config.xml2rfc_drafts:
        refs = autogen_select_refs(app)
        autogen = Autogen(app, refs)
        if app.config.xml2rfc_autogen_docs:
            autogen.gen_docs()
    return


def autogen_select_refs(app: sphinx.application.Sphinx) -> Refs:
    """Select git refs to include."""
    branch_re = re.compile(app.config.xml2rfc_autogen_branch_re)
    tag_re = re.compile(app.config.xml2rfc_autogen_tag_re)
    repo = git.Repo(path=os.path.dirname(__file__),
                    search_parent_directories=True)
    refs = {"branches": {b.name: b for b in repo.branches
                         if branch_re.match(b.name)},
            "tags": {t.name: t for t in repo.tags
                     if tag_re.match(t.name)}}
    for remote in app.config.xml2rfc_remotes:
        for ref in repo.remotes[remote].refs:
            if (branch_re.match(ref.remote_head)
                    and ref.remote_head not in refs["branches"]
                    and ref.is_detached):
                refs["branches"][ref.remote_head] = ref
    logger.debug(f"{refs=}")
    return refs


class AutoVersion(typing.NamedTuple):
    """Auto generated draft version."""

    draft: str
    ref_type: str
    ref_name: str
    ref: git.refs.reference.Reference

    @property
    def ts(self) -> datetime.datetime:
        """Get the committed date-time of the commit at `self.ref`."""
        return typing.cast(datetime.datetime,
                           self.ref.commit.committed_datetime)


class Autogen(object):
    """Document auto-generator."""

    _Items = typing.ItemsView[str, typing.Dict[str, typing.List[AutoVersion]]]

    def __init__(self, app: sphinx.application.Sphinx, refs: Refs) -> None:
        """Auto generate draft versions."""
        self.app = app
        self.base_dir = get_base_dir(self.app)
        proc = subprocess.run(("xml2rfc", "--version"),  # noqa: S603
                              capture_output=True, check=True)
        xml2rfc_version = proc.stdout.decode().strip()
        logger.info(f"sphinx-xml2rfc: using {xml2rfc_version}")

        file_names = [f"{draft}.xml" for draft in app.config.xml2rfc_drafts]
        file_names += app.config.xml2rfc_sources

        self.versions = list()
        for ref_type, _refs in refs.items():
            for ref_name, ref in _refs.items():
                output_dir = os.path.join(self.base_dir, *ref.path.split("/"))
                os.makedirs(output_dir, exist_ok=True)
                logger.debug(f"{output_dir=}")
                with tempfile.TemporaryDirectory() as tmpdir:
                    logger.debug(f"{ref_name=}, {ref.path=}")
                    for blob in ref.commit.tree.blobs:
                        if blob.name not in file_names:
                            continue
                        logger.debug(f"{blob.name=}")
                        logger.debug(f"{blob.hexsha=}")
                        with open(os.path.join(tmpdir, blob.name), "wb") as f:
                            blob.stream_data(f)
                    for draft in app.config.xml2rfc_drafts:
                        logger.info(f"generating output for {draft} "
                                    f"at {ref.path}")
                        src_path = os.path.join(tmpdir, f"{draft}.xml")
                        version = AutoVersion(draft, ref_type, ref_name, ref)
                        date = version.ts.strftime("%Y-%m-%d")
                        cmd = ("xml2rfc", src_path,
                               "--date", date,
                               "--no-pagination",
                               "--text",
                               "--path", output_dir)
                        logger.debug(f"{cmd=}")
                        try:
                            proc = subprocess.run(cmd, check=True,  # noqa: E501,S603
                                                  capture_output=True)
                        except subprocess.CalledProcessError as e:
                            logger.warning(e.stderr.decode())
                            continue
                        logger.debug(proc.stderr.decode())
                        self.versions.append(version)

    def version_items(self) -> _Items:
        """Get an `ItemsView` of a `dict` of generated versions."""
        d = {draft: {ref_type: [version for version in it]
                     for ref_type, it
                     in itertools.groupby(it, lambda v: v.ref_type)}
             for draft, it
             in itertools.groupby(sorted(self.versions), lambda v: v.draft)}
        return d.items()

    def sorted_versions(self, draft: str) -> typing.Iterator[AutoVersion]:
        """Iterate over versions of `draft` in reserve chronological order."""
        for version in sorted(self.versions, key=lambda v: v.ts, reverse=True):
            if version.draft == draft:
                yield version

    def prior_versions(self,
                       version: AutoVersion) -> typing.Iterator[AutoVersion]:
        """Iterate over prior versions of `version`."""
        for prior in self.sorted_versions(version.draft):
            if prior.ts < version.ts:
                yield prior

    def gen_docs(self) -> None:
        """Generate markdown sources for building by sphinx."""
        if not self.versions:
            return
        with open(os.path.join(self.base_dir, "toc.md"), "w",
                  encoding="utf-8") as toc_fd:
            toc_fd.write("# Internet Drafts\n\n"
                         ":::{toctree}\n"
                         ":maxdepth: 3\n\n")
            for draft, ref_types in self.version_items():
                draft_toc = f"toc-{draft}"
                toc_fd.write(f"{draft_toc}\n")
                with open(os.path.join(self.base_dir, f"{draft_toc}.md"),
                          "w", encoding="utf-8") as draft_toc_fd:
                    draft_toc_fd.write(f"# `{draft}`\n\n"
                                       f":::{{toctree}}\n\n")
                    for ref_type, versions in ref_types.items():
                        ref_type_toc = f"{draft_toc}-{ref_type}"
                        draft_toc_fd.write(f"{ref_type_toc}\n")
                        with open(os.path.join(self.base_dir, f"{ref_type_toc}.md"),  # noqa: E501
                                  "w", encoding="utf-8") as ref_type_toc_fd:
                            ref_type_toc_fd.write(f"# {ref_type}\n\n"
                                                  f":::{{toctree}}\n\n")
                            for version in versions:
                                draft_doc = os.path.join(
                                    *version.ref.path.split("/"),
                                    draft,
                                )
                                ref_type_toc_fd.write(f"{draft_doc}\n")
                                with open(os.path.join(self.base_dir, f"{draft_doc}.md"),  # noqa: E501
                                          "w", encoding="utf-8") as draft_fd:
                                    draft_fd.write(f"# {version.ref_type}: {version.ref_name}\n\n"  # noqa: E501
                                                   f":::{{xml2rfc:version}} {draft}\n"  # noqa: E501
                                                   f":ref_type: {version.ref_type}\n"  # noqa: E501
                                                   f":ref_name: {version.ref_name}\n"  # noqa: E501
                                                   f":ref_path: {version.ref.path}\n"  # noqa: E501
                                                   f":::\n")
                            ref_type_toc_fd.write(":::\n")
                    changes_toc = f"{draft_toc}-diffs"
                    draft_toc_fd.write(f"{changes_toc}\n")
                    with open(os.path.join(self.base_dir, f"{changes_toc}.md"),
                              "w", encoding="utf-8") as changes_toc_fd:
                        changes_toc_fd.write("# changes\n\n"
                                             ":::{toctree}\n\n")
                        for to_ver in self.sorted_versions(draft):
                            for from_ver in self.prior_versions(to_ver):
                                changes_doc = os.path.join(
                                    *to_ver.ref.path.split("/"),
                                    f"{draft}-diff-from-"
                                    f"{from_ver.ref_type}.{from_ver.ref_name}",
                                )
                                changes_toc_fd.write(f"{changes_doc}\n")
                                with open(os.path.join(self.base_dir, f"{changes_doc}.md"),  # noqa: E501
                                          "w", encoding="utf-8") as changes_doc_fd:  # noqa: E501
                                    changes_doc_fd.write(f"# {from_ver.ref.path} ⟼ {to_ver.ref.path}\n\n"  # noqa: E501
                                                         f":::{{xml2rfc:diff}} {draft}\n"  # noqa: E501
                                                         f":from: {from_ver.ref.path}\n"  # noqa: E501
                                                         f":to: {to_ver.ref.path}\n"  # noqa: E501
                                                         f":::\n")
                        changes_toc_fd.write(":::\n")
                    draft_toc_fd.write(":::\n")
            toc_fd.write(":::\n")
        return



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx_xml2rfc package tests."""

from __future__ import annotations

import pytest


class TestExtension:
    """Test cases for sphinx_xml2rfc."""

    @pytest.mark.sphinx("html", testroot="ext-xml2rfc", freshenv=True)
    def test_domain(self, app):
        """Test `xml2rfc` directives."""
        app.builder.build_all()
        content = (app.outdir / "index.html").read_text()
        # check tiitle
        assert "<h1>Test sphinx-xml2rfc" in content
        # check versions
        assert "Test I-D Foo" in content
        assert "Test I-D Bar" in content
        # check diff
        assert "-Test I-D Foo" in content
        assert "+Test I-D Bar" in content

    @pytest.mark.sphinx("html", testroot="ext-xml2rfc-autogen", freshenv=True)
    def test_autogen(self, test_draft_name, patched_app):  # noqa: R701
        """Test document auto-generation."""
        (app, mock_repo, mock_refs, mock_blob) = patched_app
        app.builder.build_all()
        assert mock_repo.called
        assert mock_blob.stream_data.call_count == len(mock_refs)
        outdir = app.confdir / "_xml2rfc"
        toc_root = (outdir / "toc.md").read_text()
        assert "Internet Drafts" in toc_root
        assert "toctree" in toc_root
        assert f"toc-{test_draft_name}" in toc_root
        toc_draft = (outdir / f"toc-{test_draft_name}.md").read_text()
        assert f"`{test_draft_name}`" in toc_draft
        assert "toctree" in toc_draft
        for ref_type in ("branches", "tags", "diffs"):
            assert f"toc-{test_draft_name}-{ref_type}" in toc_draft
            toc_ref_type = (outdir /
                            f"toc-{test_draft_name}-{ref_type}.md").read_text()
            assert "toctree" in toc_ref_type



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""xml2rfc document generation extension."""

from __future__ import annotations

import typing

import sphinx

from .autogen import autogen_run
from .domain import Xml2rfcDomain

logger = sphinx.util.logging.getLogger(__name__)


def setup(app: sphinx.application.Sphinx) -> typing.Dict[str, typing.Any]:
    """Sphinx extension for xml2rfc rendering."""
    app.add_config_value("xml2rfc_drafts", [], "env")
    app.add_config_value("xml2rfc_sources", [], "env")
    app.add_config_value("xml2rfc_remotes", ["origin"], "env")
    app.add_config_value("xml2rfc_autogen_docs", True, "env")
    app.add_config_value("xml2rfc_autogen_branch_re", r"^main|master$", "env")
    app.add_config_value("xml2rfc_autogen_tag_re", r"^.+$", "env")
    app.add_config_value("xml2rfc_output", "_xml2rfc", "env")

    app.add_domain(Xml2rfcDomain)

    app.connect("builder-inited", autogen_run)

    return {"version": "0.0.1",
            "parallel_read_safe": True,
            "parallel_write_safe": True}



# Copyright (c) 2021 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""sphinx-xml2rfc utils module."""

from __future__ import annotations

import os
import typing

import sphinx

if typing.TYPE_CHECKING:
    import sphinx.application

logger = sphinx.util.logging.getLogger(__name__)


def get_base_dir(app: sphinx.application.Sphinx) -> str:
    """Get the absolute path to the base output directory."""
    return os.path.join(typing.cast(str, app.confdir),
                        app.config.xml2rfc_output)
